@@ -1,9 +1,10 @@
+from __future__ import annotations
+
 import getpass
 import logging
 import re
 from argparse import Namespace
 from functools import partial
-from typing import Dict, List, Optional
 
 import attr
 import psutil
@@ -21,7 +22,6 @@ from .types import (
     TempFileInfo,
     WaitingProcess,
 )
-from .utils import clean_str
 
 logger = logging.getLogger("pgactivity")
 
@@ -71,7 +71,7 @@ class Data:
     server_encoding: bytes
     min_duration: float
     filters: Filters
-    dsn_parameters: Dict[str, str]
+    dsn_parameters: dict[str, str]
     failed_queries: FailedQueriesInfo
 
     @classmethod
@@ -79,16 +79,16 @@ class Data:
         cls,
         min_duration: float = 0.0,
         *,
-        host: Optional[str] = None,
+        host: str | None = None,
         port: int = 5432,
         user: str = "postgres",
-        password: Optional[str] = None,
+        password: str | None = None,
         database: str = "postgres",
         rds_mode: bool = False,
         dsn: str = "",
         hide_queries_in_logs: bool = False,
         filters: Filters = NO_FILTER,
-    ) -> "Data":
+    ) -> Data:
         """Create an instance by connecting to a PostgreSQL server."""
         pg_conn = pg.connect(
             dsn,
@@ -117,7 +117,7 @@ class Data:
             dsn_parameters=pg.connection_parameters(pg_conn),
         )
 
-    def try_reconnect(self) -> Optional["Data"]:
+    def try_reconnect(self) -> Data | None:
         try:
             pg_conn = pg.connect(**self.dsn_parameters)
         except (pg.InterfaceError, pg.OperationalError):
@@ -190,7 +190,7 @@ class Data:
         ret = pg.fetchone(self.pg_conn, query, {"pid": pid})
         return ret["is_stopped"]  # type: ignore[no-any-return]
 
-    def pg_get_temporary_file(self) -> Optional[TempFileInfo]:
+    def pg_get_temporary_file(self) -> TempFileInfo | None:
         """
         Count the number of temporary files and get their total size
         """
@@ -234,7 +234,7 @@ class Data:
         finally:
             pg.execute(self.pg_conn, queries.get("reset_statement_timeout"))
 
-    def pg_get_wal_senders(self) -> Optional[int]:
+    def pg_get_wal_senders(self) -> int | None:
         """
         Count the number of wal senders
         """
@@ -245,7 +245,7 @@ class Data:
         ret = pg.fetchone(self.pg_conn, query)
         return int(ret["wal_senders"])
 
-    def pg_get_wal_receivers(self) -> Optional[int]:
+    def pg_get_wal_receivers(self) -> int | None:
         """
         Count the number of wal receivers
         """
@@ -272,7 +272,7 @@ class Data:
 
         return int(ret["wal_receivers"])
 
-    def pg_get_replication_slots(self) -> Optional[int]:
+    def pg_get_replication_slots(self) -> int | None:
         """
         Count the number of replication slots
         """
@@ -291,7 +291,7 @@ class Data:
 
     def pg_get_server_information(
         self,
-        prev_server_info: Optional[ServerInformation] = None,
+        prev_server_info: ServerInformation | None = None,
         using_rds: bool = False,
         skip_db_size: bool = False,
         skip_tempfile: bool = False,
@@ -339,16 +339,17 @@ class Data:
             )
             raise
 
-        temporary_file_info: Optional[TempFileInfo] = None
+        temporary_file_info: TempFileInfo | None = None
         if not skip_tempfile:
             temporary_file_info = self.pg_get_temporary_file()
         wal_senders = self.pg_get_wal_senders()
-        wal_receivers: Optional[int] = None
+        wal_receivers: int | None = None
         if not skip_walreceiver:
             wal_receivers = self.pg_get_wal_receivers()
         replication_slots = self.pg_get_replication_slots()
 
-        hr: Optional[Pct] = None
+        hr: Pct | None = None
+        rr: Pct | None = None
         tps, ips, ups, dps, rps = 0, 0, 0, 0, 0
         size_ev = 0.0
         if prev_server_info is not None:
@@ -367,6 +368,10 @@ class Data:
                 deltahit = ret["blks_hit"] - prev_server_info.blks_hit
                 if deltaread + deltahit != 0:
                     hr = Pct(100 * deltahit / (deltaread + deltahit))
+                deltacommit = ret["xact_commit"] - prev_server_info.xact_commit
+                deltarollback = ret["xact_rollback"] - prev_server_info.xact_rollback
+                if deltacommit + deltahit > 0:
+                    rr = Pct(100 * deltarollback / (deltacommit + deltarollback))
             except ZeroDivisionError:
                 pass
 
@@ -378,6 +383,7 @@ class Data:
             delete_per_second=dps,
             tuples_returned_per_second=rps,
             cache_hit_ratio_last_snap=hr,
+            rollback_ratio_last_snap=rr,
             temporary_file=temporary_file_info,
             wal_senders=wal_senders,
             wal_receivers=wal_receivers,
@@ -385,7 +391,7 @@ class Data:
             **ret,
         )
 
-    def pg_get_activities(self, duration_mode: int = 1) -> List[RunningProcess]:
+    def pg_get_activities(self, duration_mode: int = 1) -> list[RunningProcess]:
         """
         Get activity from pg_stat_activity view.
         """
@@ -420,7 +426,7 @@ class Data:
             text_as_bytes=True,
         )
 
-    def pg_get_waiting(self, duration_mode: int = 1) -> List[WaitingProcess]:
+    def pg_get_waiting(self, duration_mode: int = 1) -> list[WaitingProcess]:
         """
         Get waiting queries.
         """
@@ -447,7 +453,7 @@ class Data:
             text_as_bytes=True,
         )
 
-    def pg_get_blocking(self, duration_mode: int = 1) -> List[BlockingProcess]:
+    def pg_get_blocking(self, duration_mode: int = 1) -> list[BlockingProcess]:
         """
         Get blocking queries
         """
@@ -506,7 +512,6 @@ class Data:
 
 def pg_connect(
     options: Namespace,
-    exit_on_failed: bool = True,
     min_duration: float = 0.0,
     filters: Filters = NO_FILTER,
 ) -> Data:
@@ -527,18 +532,10 @@ def pg_connect(
                 hide_queries_in_logs=options.hide_queries_in_logs,
             )
         except pg.OperationalError as err:
-            errmsg = str(err).strip()
-            if nb_try < 1 and (
-                isinstance(err, pg.InvalidPassword)
-                or errmsg.startswith("FATAL:  password authentication failed for user")
-                or errmsg == "fe_sendauth: no password supplied"
-            ):
+            if nb_try < 1 and pg.needs_password(err):
                 password = getpass.getpass()
-            elif exit_on_failed:
-                msg = str(err).replace("FATAL:", "")
-                raise SystemExit("pg_activity: FATAL: %s" % clean_str(msg))
             else:
-                raise Exception("Could not connect to PostgreSQL")
+                raise
         except pg.ProgrammingError as err:
             errmsg = str(err).strip()
             if errmsg.startswith("invalid dsn"):
