@@ -2,10 +2,9 @@ from __future__ import annotations
 
 import logging
 import os
+from collections.abc import Callable, Sequence
 from typing import Any, TypeVar, overload
 
-from .compat import Callable, Dict, Sequence
-
 Row = TypeVar("Row")
 
 try:
@@ -29,7 +28,7 @@ try:
 
     __version__ = psycopg.__version__
 
-    Connection = psycopg.Connection[Dict[str, Any]]
+    Connection = psycopg.Connection[dict[str, Any]]
 
     class BytesLoader(Loader):
         def load(self, data: Buffer) -> bytes:
@@ -49,7 +48,7 @@ try:
             # This is (more or less) what's done by psql.
             conninfo = conninfo_to_dict(dsn)
             conninfo.setdefault("client_encoding", "auto")
-            dsn = make_conninfo(**conninfo)
+            dsn = make_conninfo("", **conninfo)
         conn = psycopg.connect(dsn, autocommit=True, row_factory=dict_row, **kwargs)
         if conn.info.encoding == "ascii":
             # If client encoding is still 'ascii', fall back to a loader with a replace
