@@ -1,12 +1,10 @@
 -- Get data from pg_activity before pg 9.2
 SELECT
       a.procpid AS pid,
+      NULL AS xmin,
       '<unknown>' AS application_name,
       a.datname AS database,
-      CASE WHEN a.client_addr IS NULL
-          THEN 'local'
-          ELSE a.client_addr::TEXT
-      END AS client,
+      a.client_addr AS client,
       EXTRACT(epoch FROM (NOW() - a.{duration_column})) AS duration,
       a.waiting AS wait,
       a.usename AS user,
@@ -18,8 +16,9 @@ SELECT
       END AS state,
       CASE
           WHEN a.current_query LIKE '<IDLE>%%' THEN NULL
-          ELSE convert_from(replace(a.current_query, '\', '\\')::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8'))
+          ELSE a.current_query
       END AS query,
+      pg_catalog.pg_encoding_to_char(b.encoding) AS encoding,
       NULL AS query_leader_pid,
       false AS is_parallel_worker
   FROM
@@ -28,11 +27,11 @@ SELECT
  WHERE
       current_query <> '<IDLE>'
   AND procpid <> pg_backend_pid()
-  AND CASE WHEN %(min_duration)s = 0
+  AND CASE WHEN {min_duration} = 0
           THEN true
           ELSE extract(epoch from now() - {duration_column}) > %(min_duration)s
       END
-  AND CASE WHEN %(dbname_filter)s IS NULL THEN true
+  AND CASE WHEN {dbname_filter} IS NULL THEN true
       ELSE a.datname ~* %(dbname_filter)s
       END
 ORDER BY
