@@ -3,17 +3,16 @@
 -- NEW pg_stat_activity.waiting => pg_stat_activity.wait_event
 SELECT
       a.pid AS pid,
+      a.backend_xmin AS xmin,
       a.application_name AS application_name,
       a.datname AS database,
-      CASE WHEN a.client_addr IS NULL
-          THEN 'local'
-          ELSE a.client_addr::TEXT
-      END AS client,
+      a.client_addr AS client,
       EXTRACT(epoch FROM (NOW() - a.{duration_column})) AS duration,
       a.wait_event AS wait,
       a.usename AS user,
       a.state AS state,
-      convert_from(replace(a.query, '\', '\\')::bytea, coalesce(pg_catalog.pg_encoding_to_char(b.encoding), 'UTF8')) AS query,
+      a.query AS query,
+      pg_catalog.pg_encoding_to_char(b.encoding) AS encoding,
       NULL AS query_leader_pid,
       false AS is_parallel_worker
   FROM
@@ -22,11 +21,11 @@ SELECT
  WHERE
       state <> 'idle'
   AND pid <> pg_backend_pid()
-  AND CASE WHEN %(min_duration)s = 0
+  AND CASE WHEN {min_duration} = 0
           THEN true
           ELSE extract(epoch from now() - {duration_column}) > %(min_duration)s
       END
-  AND CASE WHEN %(dbname_filter)s IS NULL THEN true
+  AND CASE WHEN {dbname_filter} IS NULL THEN true
       ELSE a.datname ~* %(dbname_filter)s
       END
 ORDER BY
