@@ -1,5 +1,6 @@
+from __future__ import annotations
+
 import time
-from typing import Optional
 
 import attr
 import psycopg
@@ -10,28 +11,26 @@ from pgactivity import types
 from pgactivity.data import Data
 
 
-def wait_for_data(fct, msg: str, timeout: int = 2):
+def retry(fct, msg: str, timeout: int = 2):
     count = int(timeout / 0.1)
     for _ in range(count):
         time.sleep(0.1)
-        data = fct()
-
-        if not data:
-            continue
-        break
-    else:
-        raise AssertionError(msg)
-    return data
+        value = fct()
+        if value:
+            return value
+    pytest.fail(msg)
 
 
 @pytest.fixture
 def data(postgresql):
-    return Data.pg_connect(
+    obj = Data.pg_connect(
         host=postgresql.info.host,
         port=postgresql.info.port,
         database=postgresql.info.dbname,
         user=postgresql.info.user,
     )
+    yield obj
+    obj.pg_conn.close()
 
 
 def test_pg_is_local(postgresql, data):
@@ -64,9 +63,7 @@ def test_blocking_waiting(postgresql, data, execute):
     execute("UPDATE t SET s = 'blocking'")
     execute("UPDATE t SET s = 'waiting 1'", commit=True)
     execute("UPDATE t SET s = 'waiting 2'", commit=True)
-    blocking = wait_for_data(
-        data.pg_get_blocking, msg="could not fetch blocking queries"
-    )
+    blocking = retry(data.pg_get_blocking, msg="could not fetch blocking queries")
     waiting = data.pg_get_waiting()
     assert len(blocking) == 2
     assert len(waiting) == 2
@@ -84,9 +81,7 @@ def test_pg_get_blocking_virtualxid(postgresql, data, execute):
     postgresql.commit()
     execute("UPDATE t SET s = 'blocking'")
     execute("CREATE INDEX CONCURRENTLY ON t(s)", autocommit=True)
-    (blocking,) = wait_for_data(
-        data.pg_get_blocking, msg="could not fetch blocking queries"
-    )
+    (blocking,) = retry(data.pg_get_blocking, msg="could not fetch blocking queries")
     (waiting,) = data.pg_get_waiting()
     assert "blocking" in blocking.query
     assert "CREATE INDEX CONCURRENTLY ON t(s)" in waiting.query
@@ -103,7 +98,7 @@ def test_terminate_backend(postgresql, data):
     postgresql.execute("SELECT pg_sleep(1)")
     (running,) = data.pg_get_activities()
     assert data.pg_terminate_backend(running.pid)
-    assert not data.pg_get_activities()
+    retry(lambda: not data.pg_get_activities(), msg="some processes are still active")
 
 
 def test_encoding(postgresql, data, execute):
@@ -122,7 +117,7 @@ def test_encoding(postgresql, data, execute):
             else:
                 break
         else:
-            pytest.fail(
+            pytest.skip(
                 f"could not create a database with encoding amongst {', '.join(encodings)}"
             )
 
@@ -132,9 +127,9 @@ def test_encoding(postgresql, data, execute):
 
     execute("UPDATE tbl SET s = 'blocking éléphant'", dbname="latin1")
     execute("UPDATE tbl SET s = 'waiting éléphant'", dbname="latin1", commit=True)
-    running = wait_for_data(data.pg_get_activities, msg="could not fetch activities")
+    running = retry(data.pg_get_activities, msg="could not fetch activities")
     assert "blocking éléphant" in running[0].query
-    (waiting,) = wait_for_data(data.pg_get_waiting, "no waiting process")
+    (waiting,) = retry(data.pg_get_waiting, "no waiting process")
     assert waiting.query and "waiting éléphant" in waiting.query
     (blocking,) = data.pg_get_blocking()
     assert blocking.query and "blocking éléphant" in blocking.query
@@ -149,9 +144,12 @@ def test_client_encoding(postgresql, encoding: str) -> None:
         user=postgresql.info.user,
         dsn=f"client_encoding={encoding}",
     )
-    assert data.pg_version.startswith(
-        f"PostgreSQL {str(postgresql.info.server_version)[:2]}"
-    )
+    try:
+        assert data.pg_version.startswith(
+            f"PostgreSQL {str(postgresql.info.server_version)[:2]}"
+        )
+    finally:
+        data.pg_conn.close()
 
 
 @pytest.mark.parametrize(
@@ -163,10 +161,13 @@ def test_client_encoding(postgresql, encoding: str) -> None:
     ],
 )
 def test_postgres_and_python_encoding(
-    database_factory, pyenc: str, pgenc: str, locale: Optional[str], data, postgresql
+    database_factory, pyenc: str, pgenc: str, locale: str | None, data, postgresql
 ) -> None:
     dbname = pyenc
-    database_factory(dbname, encoding=pgenc, locale=locale)
+    try:
+        database_factory(dbname, encoding=pgenc, locale=locale)
+    except WrongObjectType:
+        pytest.skip(f"could not create a database with encoding '{pgenc}'")
     with psycopg.connect(
         postgresql.info.dsn, dbname=dbname, client_encoding="utf-8"
     ) as conn:
@@ -180,11 +181,11 @@ def test_postgres_and_python_encoding(
 def test_filters_dbname(data, execute):
     data_filtered = attr.evolve(data, filters=types.Filters(dbname="temp"))
     execute("SELECT pg_sleep(2)", dbname="template1", autocommit=True)
-    nbconn = wait_for_data(
+    nbconn = retry(
         data.pg_get_server_information,
         msg="could not get active connections for filtered DBNAME",
     )
-    nbconn_filtered = wait_for_data(
+    nbconn_filtered = retry(
         data_filtered.pg_get_server_information,
         msg="could not get active connections for filtered DBNAME",
     )
